==> nui_core-0.0.69.tar.gz: lib/ui/component/pageview/nui_pageview.dart <==
import 'dart:developer';
import 'package:flutter/cupertino.dart';
import 'package:nui_core/bloc/nui_bloc.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/ui/component/listview/nui_list_item_repo.dart';
import 'package:nui_core/ui/component/listview/nui_list_models.dart';
import 'package:nui_core/ui/component/listview/nui_listview_bloc.dart';
import 'package:nui_core/ui/component/listview/nui_listview_functions.dart';
import 'package:nui_core/ui/component/listview/nui_listview_widgets.dart';

typedef OnPageChangeListener = void Function(int page);

class NUIPageView<T, C> extends StatefulWidget {
  NUIPageView({@required this.bloc, @required this.repo, @required this.initialList, @required this.clickListener, this.pageChangeListener, this.dividerPadding, this.initialPosition: 0, this.viewportFraction: 0.8, this.skeletalCount : 0, this.fixedFirstItem, this.fixedLastItem, this.withItemCount:true, this.cacheOffset, this.enablePagination: false, this.minimumPaginateCount: 5, this.skeletalInflater, this.paginationInflater, this.placeHolder, this.errorHolder, this.checkedMap, this.configMap});
  final NUIListItemRepository repo;
  final List<T> initialList;
  final bool withItemCount;
  final double cacheOffset;
  final int initialPosition;

  final OnPageChangeListener pageChangeListener;
  final double dividerPadding;

  final ListFixedItemGetter fixedFirstItem;
  final ListFixedItemGetter fixedLastItem;

  final double viewportFraction;

  final Map<String, dynamic> checkedMap;
  final Map<String, dynamic> configMap;

  //Click callback
  final OnListItemClickListener clickListener;

  //Loader widgets
  final int skeletalCount;

  //Widget functions
  final ListErrorHolderInflater<T, C> errorHolder;
  final ListPlaceHolderInflater placeHolder;
  final ListLoadingInflater skeletalInflater;
  final ListLoadingInflater paginationInflater;

  final bool enablePagination;
  final int minimumPaginateCount;

  final NUIListViewBloc<T, C> bloc;
  @override
  _NUIPageViewState<T, C> createState() => _NUIPageViewState<T, C>();
}

class _NUIPageViewState<T, C> extends State<NUIPageView> {
  PageController _pageController;
  double _offset = 0.0;
  NUIListViewModel<T, C> curModel;
  NUIListViewBloc<T, C> bloc;
  List items = List();
  NUIListViewParentHandler actionHandler;
  Map<String, dynamic> checkedMap;
  Map<String, dynamic> configMap;

  bool isFirst = true;

  final PaginationLoad pagination = PaginationLoad();

  bool _hasSkeletalLoad(){
    return widget.skeletalInflater != null && widget.skeletalCount > 0;
  }

  @override
  void initState(){
    super.initState();
    checkedMap = widget.checkedMap ?? Map();
    configMap = widget.configMap ?? Map();
    bloc = widget.bloc;
    curModel = NUIListViewModel<T, C>(data: widget.initialList ?? List());
    if(!isNullOrEmpty(widget.initialList)){
      items.addAll(widget.initialList);
    }
    _pageController = new PageController(viewportFraction: widget.viewportFraction ?? 0.8, initialPage: widget.initialPosition ?? 0);
    _pageController.addListener(() {
      if (_pageController.position.maxScrollExtent == _pageController.position.pixels) {
        log("NUIPageViewBuilder -> running pagination loading ... ");
        _onPaginationLoad(cursor: curModel.cursor);
      }
    });

    actionHandler = NUIListViewParentHandler(widget.clickListener, checkedMap, configMap, (){

    }, (){

    });
  }

  dynamic _getItem(int position){
    return items[position];
  }

  Widget _renderListItem(dynamic data, int position){
    final padding = EdgeInsets.only(left: widget.dividerPadding ?? 0, right: widget.dividerPadding ?? 0);

    if(data is PaginationLoad){
      //Pagination loading
      return Padding(
        padding: padding,
        child: widget.paginationInflater != null ? widget.paginationInflater(position) : NUIListViewProgressWidget(),
      );
    }
    else if(data == null){
      return Padding(
        padding: padding,
        child: widget.skeletalInflater != null ? widget.skeletalInflater(position) : NUIListViewProgressWidget(),
      );
    }

    NUIListItem item = widget.repo.getItemByClass(data.runtimeType);
    if(item == null){
      throw Exception("NUIPageViewBuilder -> List item for type : ${data.runtimeType.toString()} is not registered in the repository");
    }

    NUIListActionController handler = NUIListActionController(actionHandler, NUIListItemModel(position, data, checkedMap: checkedMap, configMap: configMap));
    return Padding(
      padding: padding,
      child: item.inflater(data, position, handler),
    );
  }

  Widget _getPageViewBuilder(){
    logNUI("NUIPageView", "Using  itemCount: ${widget.withItemCount ? items.length : null}, length : ${items.length}");
    final builder = PageView.builder(
        controller: _pageController,
        onPageChanged: (index) {
          if(widget.pageChangeListener != null) widget.pageChangeListener(index);
        },
        itemCount: widget.withItemCount ? items.length : null,
        itemBuilder: (context, position){
          if(position < items.length) {
            return _renderListItem(_getItem(position), position);
          }
          return null;
        });

    return builder;
  }

  void _onPaginationLoad({bool isFirst, C cursor}){
    if (isFirst == true || (widget.enablePagination == true && !curModel.isLoading())) {
      if(isFirst != true && curModel.data != null && curModel.data.length < widget.minimumPaginateCount) {
        logNUI("NUIPageViewBuilder", "Minimum pagination threshold of ${widget.minimumPaginateCount} has not been reached, current count at ${curModel.data.length}");
        return;
      }
      logNUI("NUIPageViewBuilder", "Initiating pagination loading with cursor : ${cursor != null}, is first : $isFirst");
      curModel.paginationLoading = true;
      bloc.startPaginationLoading(isFirst, curModel.data, cursor);
    }
  }

  void _onSkeletalLoad(){
    if (!curModel.isLoading()) {
      curModel.skeletalLoading = true;
      bloc.startSkeletalLoading();
    }
  }

  Widget _buildList(){
    return NotificationListener(
      child: _getPageViewBuilder(),
      onNotification: (notification){
        if(notification is ScrollNotification){
          _offset = notification.metrics.pixels;
        }
        return true;
      },
    );
  }

  Widget _buildStreamBuilder(){
    return StreamBuilder(
      initialData: NUIListViewModel<T, C>(paginationLoading: true, data: curModel.data, message: null),
      stream: bloc.stream,
      builder: (BuildContext context, AsyncSnapshot snapshot) {
        NUIListViewModel<T, C> model = snapshot.data;

        //Update checked list and config list if specified
        if(model.updateCheckedMap != null) model.updateCheckedMap(checkedMap);
        logNUI("NUIPageView", "Checked keys : ${checkedMap.keys}");
        if(model.updateConfigMap != null) model.updateConfigMap(configMap);

        logNUI("NUIPageView", "Setting new list model for list ${T.toString()} with cursor type ${C.toString()}");
        curModel = model;
        if (model.paginationLoading) {
          // Handle progress state
          items.remove(pagination);
          items.add(pagination);
          return _buildList();
        }
        else if(model.skeletalLoading){
          items.clear();
          //Add number of skeletal (null) models into the list according to the skeletal count specified
          for(int i = 0; i < widget.skeletalCount; i++){
            items.add(null);
          }
          return _buildList();
        }
        else if(model.success == false && isNullOrEmpty(model.data) && widget.errorHolder != null){
          return widget.errorHolder(model);
        }
        else if(isNullOrEmpty(model.data) && widget.placeHolder != null){
          return widget.placeHolder();
        }
        else {
          // Handle data state
          items.clear();
          if(widget.fixedFirstItem != null){
            items.add(widget.fixedFirstItem());
          }
          items.addAll(model.data);
          if(widget.fixedLastItem != null){
            items.add(widget.fixedLastItem());
          }
          return _buildList();
        }
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    if(_hasSkeletalLoad()){
      _onSkeletalLoad();
    }
    else{
      _onPaginationLoad(isFirst: isFirst, cursor: curModel.cursor);
      isFirst = false;
    }
    return NUIBlocWidget(
        bloc: bloc,
        child: _buildStreamBuilder()
    );
  }
}

==> nui_core-0.0.70.tar.gz: lib/ui/component/pageview/nui_pageview.dart <==
import 'dart:developer';
import 'package:flutter/cupertino.dart';
import 'package:nui_core/bloc/nui_bloc.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/listview/nui_list_item_repo.dart';
import 'package:nui_core/ui/component/listview/nui_list_models.dart';
import 'package:nui_core/ui/component/listview/nui_listview_bloc.dart';
import 'package:nui_core/ui/component/listview/nui_listview_functions.dart';
import 'package:nui_core/ui/component/listview/nui_listview_widgets.dart';
import 'package:nui_core/ui/component/nui_state_widget.dart';

typedef OnPageChangeListener = void Function(int page);
typedef NUIPageViewPageCallback = void Function(int page);
typedef NUIPageViewItemsCallback = void Function(List<dynamic> items);

class NUIPageViewController{
  NUIPageViewPageCallback _changePageCallback;
  NUIPageViewItemsCallback _updateItemsCallback;
  void changePage(int page){
    if(_changePageCallback != null) _changePageCallback(page);
  }
  void updateItems(List<dynamic> items){
    if(_updateItemsCallback != null) _updateItemsCallback(items);
  }
  void _dispose(){
    _changePageCallback = null;
    _updateItemsCallback = null;
  }
}

class NUIPageView<T, C> extends StatefulWidget {
  NUIPageView({@required this.bloc, @required this.repo, @required this.initialList, this.controller, @required this.clickListener, this.pageChangeListener, this.dividerPadding, this.initialPosition: 0, this.viewportFraction: 0.8, this.skeletalCount : 0, this.fixedFirstItem, this.fixedLastItem, this.withItemCount:true, this.cacheOffset, this.enablePagination: false, this.minimumPaginateCount: 5, this.skeletalInflater, this.paginationInflater, this.placeHolder, this.errorHolder, this.checkedMap, this.configMap});
  final NUIListItemRepository repo;
  final List<T> initialList;
  final bool withItemCount;
  final double cacheOffset;
  final int initialPosition;

  final NUIPageViewController controller;

  final OnPageChangeListener pageChangeListener;
  final double dividerPadding;

  final ListFixedItemGetter fixedFirstItem;
  final ListFixedItemGetter fixedLastItem;

  final double viewportFraction;

  final Map<String, dynamic> checkedMap;
  final Map<String, dynamic> configMap;

  //Click callback
  final OnListItemClickListener clickListener;

  //Loader widgets
  final int skeletalCount;

  //Widget functions
  final ListErrorHolderInflater<T, C> errorHolder;
  final ListPlaceHolderInflater placeHolder;
  final ListLoadingInflater skeletalInflater;
  final ListLoadingInflater paginationInflater;

  final bool enablePagination;
  final int minimumPaginateCount;

  final NUIListViewBloc<T, C> bloc;
  @override
  _NUIPageViewState<T, C> createState() => _NUIPageViewState<T, C>();
}

class _NUIPageViewState<T, C> extends State<NUIPageView> {
  String _instanceId = randomUUID();
  PageController _pageController;
  double _offset = 0.0;
  NUIListViewModel<T, C> curModel;
  NUIListViewBloc<T, C> bloc;
  List items = List();
  NUIListViewParentHandler actionHandler;
  Map<String, dynamic> checkedMap;
  Map<String, dynamic> configMap;
  NUIPageViewController controller;

  bool isFirst = true;

  final PaginationLoad pagination = PaginationLoad();

  bool _hasSkeletalLoad(){
    return widget.skeletalInflater != null && widget.skeletalCount > 0;
  }

  @override
  void initState(){
    super.initState();
    controller = widget.controller ?? NUIPageViewController();
    controller._changePageCallback = (page){
    };
    controller._updateItemsCallback = (items) {
      this.items = items;
      updateNUIState<List<dynamic>>(id: _instanceId, update: items);
    };

    checkedMap = widget.checkedMap ?? Map();
    configMap = widget.configMap ?? Map();
    bloc = widget.bloc;
    curModel = NUIListViewModel<T, C>(data: widget.initialList ?? List());
    if(!isNullOrEmpty(widget.initialList)){
      items.addAll(widget.initialList);
    }
    _pageController = new PageController(viewportFraction: widget.viewportFraction ?? 0.8, initialPage: widget.initialPosition ?? 0);
    _pageController.addListener(() {
      if (_pageController.position.maxScrollExtent == _pageController.position.pixels) {
        log("NUIPageViewBuilder -> running pagination loading ... ");
        _onPaginationLoad(cursor: curModel.cursor);
      }
    });

    actionHandler = NUIListViewParentHandler(widget.clickListener, checkedMap, configMap, (){

    }, (){

    });
  }

  dynamic _getItem(int position){
    return items[position];
  }

  Widget _renderListItem(dynamic data, int position){
    final padding = EdgeInsets.only(left: widget.dividerPadding ?? 0, right: widget.dividerPadding ?? 0);

    if(data is PaginationLoad){
      //Pagination loading
      return Padding(
        padding: padding,
        child: widget.paginationInflater != null ? widget.paginationInflater(position) : NUIListViewProgressWidget(),
      );
    }
    else if(data == null){
      return Padding(
        padding: padding,
        child: widget.skeletalInflater != null ? widget.skeletalInflater(position) : NUIListViewProgressWidget(),
      );
    }

    NUIListItem item = widget.repo.getItemByClass(data.runtimeType);
    if(item == null){
      throw Exception("NUIPageViewBuilder -> List item for type : ${data.runtimeType.toString()} is not registered in the repository");
    }

    NUIListActionController handler = NUIListActionController(actionHandler, NUIListItemModel(position, data, checkedMap: checkedMap, configMap: configMap));
    return Padding(
      padding: padding,
      child: item.inflater(data, position, handler),
    );
  }

  Widget _getPageViewBuilder(List<dynamic> newItems){
    logNUI("NUIPageView", "Using  itemCount: ${widget.withItemCount ? newItems.length : null}, length : ${newItems.length}");
    final builder = PageView.builder(
        controller: _pageController,
        onPageChanged: (index) {
          if(widget.pageChangeListener != null) widget.pageChangeListener(index);
        },
        itemCount: widget.withItemCount ? newItems.length : null,
        itemBuilder: (context, position){
          if(position < newItems.length) {
            return _renderListItem(_getItem(position), position);
          }
          return null;
        });

    return builder;
  }

  void _onPaginationLoad({bool isFirst, C cursor}){
    if (isFirst == true || (widget.enablePagination == true && !curModel.isLoading())) {
      if(isFirst != true && curModel.data != null && curModel.data.length < widget.minimumPaginateCount) {
        logNUI("NUIPageViewBuilder", "Minimum pagination threshold of ${widget.minimumPaginateCount} has not been reached, current count at ${curModel.data.length}");
        return;
      }
      logNUI("NUIPageViewBuilder", "Initiating pagination loading with cursor : ${cursor != null}, is first : $isFirst");
      curModel.paginationLoading = true;
      bloc.startPaginationLoading(isFirst, curModel.data, cursor);
    }
  }

  void _onSkeletalLoad(){
    if (!curModel.isLoading()) {
      curModel.skeletalLoading = true;
      bloc.startSkeletalLoading();
    }
  }

  Widget _buildList(){
    return NUIStateful<List<dynamic>>(
      id: _instanceId,
      initial: items,
      child: (newList) => NotificationListener(
        child: _getPageViewBuilder(newList),
        onNotification: (notification){
          if(notification is ScrollNotification){
            _offset = notification.metrics.pixels;
          }
          return true;
        },
      ),
    );
  }

  Widget _buildStreamBuilder(){
    return StreamBuilder(
      initialData: NUIListViewModel<T, C>(paginationLoading: true, data: curModel.data, message: null),
      stream: bloc.stream,
      builder: (BuildContext context, AsyncSnapshot snapshot) {
        NUIListViewModel<T, C> model = snapshot.data;

        //Update checked list and config list if specified
        if(model.updateCheckedMap != null) model.updateCheckedMap(checkedMap);
        logNUI("NUIPageView", "Checked keys : ${checkedMap.keys}");
        if(model.updateConfigMap != null) model.updateConfigMap(configMap);

        logNUI("NUIPageView", "Setting new list model for list ${T.toString()} with cursor type ${C.toString()}");
        curModel = model;
        if (model.paginationLoading) {
          // Handle progress state
          items.remove(pagination);
          items.add(pagination);
          return _buildList();
        }
        else if(model.skeletalLoading){
          items.clear();
          //Add number of skeletal (null) models into the list according to the skeletal count specified
          for(int i = 0; i < widget.skeletalCount; i++){
            items.add(null);
          }
          return _buildList();
        }
        else if(model.success == false && isNullOrEmpty(model.data) && widget.errorHolder != null){
          return widget.errorHolder(model);
        }
        else if(isNullOrEmpty(model.data) && widget.placeHolder != null){
          return widget.placeHolder();
        }
        else {
          // Handle data state
          items.clear();
          if(widget.fixedFirstItem != null){
            items.add(widget.fixedFirstItem());
          }
          items.addAll(model.data);
          if(widget.fixedLastItem != null){
            items.add(widget.fixedLastItem());
          }
          return _buildList();
        }
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    if(_hasSkeletalLoad()){
      _onSkeletalLoad();
    }
    else{
      _onPaginationLoad(isFirst: isFirst, cursor: curModel.cursor);
      isFirst = false;
    }
    return NUIBlocWidget(
        bloc: bloc,
        child: _buildStreamBuilder()
    );
  }

  @override
   void dispose(){
    super.dispose();
    controller._dispose();
  }
}
